import isString from 'lodash/isString';
import partial from 'lodash/fp/partial';
import request, { makeParams } from '../utils/request';
import post from '../utils/post';
import doDelete from '../utils/delete';
import config from '../utils/config';
import { getDomain } from '../utils/auth';

export const interfaceApi = (coordinate) => {
  const { containerType, containerName, interfaceName } = isString(coordinate) ? {
    containerType: 'global',
    interfaceName: coordinate,
  } : (coordinate || {});
  if (containerType === 'global') {
    return {
      invoke: partial(userApi.invokeInterface, [interfaceName]),
      validate: partial(userApi.validateState, [interfaceName]),
    };
  } else if (containerType === 'module') {
    return {
      invoke: partial(userApi.invokeModuleInterface, [containerName, interfaceName]),
      validate: partial(userApi.validateModuleInterface, [containerName, interfaceName]),
    };
  } else {
    throw new Error(`Unsupported containerType: ${containerType}`);
  }
};

export const userApi = {
  async getAllInterfaceInfoes() {
    return request(`${config.apiContextPath}/api/interface/user/info`);
  },
  async getInterfaceInfo(name) {
    return request(`${config.apiContextPath}/api/interface/user/${encodeURIComponent(name)}/info`);
  },
  async validateState(name, params, dmPath) {
    return request(`${config.apiContextPath}/api/interface/user/${encodeURIComponent(name)}/invoke/validateState`, makeParams({ dmPath }, { params }));
  },
  async invokeInterface(name, params, dmPath) {
    return post(`${config.apiContextPath}/api/interface/user/${encodeURIComponent(name)}/invoke`, { params, dmPath });
  },
  async validateModuleInterface(mdName, name, params, dmPath) {
    return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/interface/${encodeURIComponent(name)}/invoke/validateState`, makeParams({ dmPath }, { params }));
  },
  async invokeModuleInterface(mdName, name, params, dmPath) {
    return post(`${config.apiContextPath}/api/module/user/${encodeURIComponent(mdName)}/interface/${encodeURIComponent(name)}/invoke`, { params, dmPath });
  },
  async getInterfaceDocument(name, dmPath) {
    return request(`${config.apiContextPath}/api/interface/user/${encodeURIComponent(name)}/document`, { dmPath });
  },
};

export const adminApi = {
  async getAllInterfaceInfoes(dmPath) {
    return request(`${config.apiContextPath}/api/interface/admin/info`, { dmPath });
  },
  async getInterfaceInfo(name, dmPath) {
    return request(`${config.apiContextPath}/api/interface/admin/${encodeURIComponent(name)}/info`, { dmPath });
  },
  async getInterfaceConfigure(name, dmPath) {
    return request(`${config.apiContextPath}/api/interface/admin/${encodeURIComponent(name)}/configure`, { dmPath });
  },
  async getInterfacePlainConfigure(name, dmPath) {
    return request(`${config.apiContextPath}/api/interface/admin/${encodeURIComponent(name)}/plainConfigure`, { dmPath });
  },
  async getInterfacePlainConfigures(dmPath) {
    return request(`${config.apiContextPath}/api/interface/admin/plainConfigures`, { dmPath });
  },
  async setInterfaceConfigure(name, configure, dmPath) {
    return post(`${config.apiContextPath}/api/interface/admin/${encodeURIComponent(name)}/configure`, { configure, dmPath });
  },
  async setInterfaceConfigures(configure, dmPath) {
    return post(`${config.apiContextPath}/api/interface/admin/configures`, { configure, dmPath });
  },
  async removeInterfaceConfigure(name, dmPath) {
    return doDelete(`${config.apiContextPath}/api/interface/admin/${encodeURIComponent(name)}/configure`, { dmPath });
  },
  async getInterfaceDocument(name, dmPath) {
    return request(`${config.apiContextPath}/api/interface/admin/${encodeURIComponent(name)}/document`, { dmPath });
  },
  async setInterfaceDocument(name, document, dmPath) {
    return post(`${config.apiContextPath}/api/interface/admin/${encodeURIComponent(name)}/document`, { document, dmPath });
  },
};

export class Interface {
  constructor(coordinate, dmPath) {
    this.api = interfaceApi(coordinate);
    this.dmPath = dmPath;
    this.dealWithPath = ::this.dealWithPath;
    this.invoke = ::this.invoke;
    this.validate = ::this.validate;
  }
  async dealWithPath() {
    if (this.dmPath) {
      return this.dmPath;
    } else {
      const domain = await getDomain();
      return domain.path;
    }
  }
  async invoke(params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.invoke(params, dmPath);
  }
  async validate(params = {}) {
    const dmPath = await this.dealWithPath();
    return this.api.validate(params, dmPath);
  }
}
