/* eslint-disable no-underscore-dangle,no-param-reassign */
import { connect } from 'dva';
import { shallowEqual } from './helper';

const registerModel = (app, model) => {
  // noinspection JSUnresolvedVariable
  if (app._models.filter(m => m.namespace === model.namespace).length === 1) {
    // eslint-disable-next-line no-console
    console.warn(`model: ${model.namespace} exist! unmodel it and remodeled.`);
    app.unmodel(model.namespace);
  }
  app.model(model);
};

const normLocalState = (preState, state) => {
  if (!preState || !state) {
    return state;
  }
  const { loading: preLoading, ...preModels } = preState;
  const { loading, ...models } = state;
  if (shallowEqual(preModels, models)) {
    if (!preLoading && !loading) {
      return preState;
    }
    if (!preLoading || !loading) {
      return state;
    }
    const { global: preGlobal, models: preModelsLoading, effects: preEffectsLoading } = preLoading;
    const { global, models: modelsLoading, effects: effectsLoading } = loading;
    if ((preGlobal && !global) || (!preGlobal && global)) {
      return state;
    }
    return (shallowEqual(preModelsLoading, modelsLoading) && shallowEqual(preEffectsLoading, effectsLoading)) ? preState : state;
  } else {
    return state;
  }
};

const hackDispatch = (module, dispatch) => action => dispatch({
  ...action,
  type: `${module}/${action.type}`,
});

const hackSagaEffects = (module, sagaEffects) => {
  // const put = (action) => {
  //   const { type } = action;
  //   return sagaEffects.put({ ...action, type: `${module}/${type}` });
  // };
  // put.resolve = (action) => {
  //   const { type } = action;
  //   return sagaEffects.put.resolve({ ...action, type: `${module}/${type}` });
  // };
  const select = (selector, ...args) => {
    const _selector = (state, ..._args) => {
      const keys = Object.keys(state);
      const newState = { ...state };
      for (const key of keys) {
        if (key.startsWith(`${module}/`)) {
          newState[key.substring(module.length + 1)] = newState[key];
          delete newState[key];
        }
      }
      return selector(newState, ..._args);
    };
    return sagaEffects.select(_selector, ...args);
  };
  // const take = (type) => {
  //   const { take: oTake } = sagaEffects;
  //   if (typeof type === 'string') {
  //     return oTake(`${module}/${type}`);
  //   } else if (Array.isArray(type)) {
  //     return oTake(type.map((t) => {
  //       if (typeof t === 'string') {
  //         return `${module}/${type}`;
  //       }
  //       return t;
  //     }));
  //   } else {
  //     return oTake(type);
  //   }
  // };
  return { ...sagaEffects, select };
};

const hackEffect = (module, effect) => function * effectGenerator(action, sagaEffects) {
  return yield effect(action, hackSagaEffects(module, sagaEffects));
};

const hackEffects = (module, effects) => {
  const hackedEffects = {};
  for (const key of Object.keys(effects)) {
    const effect = effects[key];
    if (Array.isArray(effect)) {
      hackedEffects[key] = [hackEffect(module, effect[0]), effect[1]];
    } else {
      hackedEffects[key] = hackEffect(module, effect);
    }
  }
  return hackedEffects;
};

const hackSubscriptions = (module, subscriptions) => {
  const hackedSubscriptions = {};
  for (const key of Object.keys(subscriptions)) {
    const subscription = subscriptions[key];
    hackedSubscriptions[key] = (api, done) => subscription(
      { ...api, dispatch: hackDispatch(module, api.dispatch) },
      done,
    );
  }
  return hackedSubscriptions;
};

export const hackModel = (module, model) => {
  model = { ...model };
  model.namespace = `${module}/${model.namespace}`;
  model.initialState = { ...model.state };
  model.reducers = model.reducers || {};
  model.reducers['@@reset'] = () => model.initialState;
  model.effects = hackEffects(module, model.effects || {});
  model.subscriptions = hackSubscriptions(module, model.subscriptions || {});
  return model;
};

export const bindModel = (app, info, layout) => (...models) => {
  const _models = [];
  for (let model of models) {
    if (typeof model === 'function') {
      model = hackModel(info.name, model(info, layout));
    } else {
      model = hackModel(info.name, model);
    }
    _models.push(model);
    registerModel(app, model);
  }
  const getLocalNamespace = ns => ns.substring(info.name.length + 1);
  return (mapStateToProps, mapDispatchToProps, mergeProps) => (route) => {
    let preLocalState = {};
    const createLocalState = (state) => {
      const localState = {};
      for (const model of _models) {
        const localNamespace = getLocalNamespace(model.namespace);
        localState[localNamespace] = state[model.namespace];
        if (state.loading) {
          localState.loading = {};
          if (state.loading.models) {
            localState.loading.models = {};
            localState.loading.models[localNamespace] = state.loading.models[model.namespace];
          }
          if (state.loading.effects) {
            localState.loading.effects = {};
            const effects = state.loading.effects;
            for (const key of Object.keys(effects)) {
              if (key.startsWith(`${model.namespace}/`)) {
                localState.loading.effects[getLocalNamespace(key)] = state.loading.effects[key];
              }
            }
            localState.loading.global = Object.values(localState.loading.models).some(e => e)
              || Object.values(localState.loading.effects).some(e => e);
          }
        }
      }
      return localState;
    };

    if (!mapStateToProps) {
      mapStateToProps = localState => localState;
    }
    const _mapStateToProps = (state, ownProps) => {
      preLocalState = normLocalState(preLocalState, createLocalState(state));
      return mapStateToProps(preLocalState, state, ownProps);
    };
    if (!mapDispatchToProps) {
      mapDispatchToProps = dispatch => ({ dispatch });
    }
    const _mapDispatchToProps = (dispatch, ownProps) => mapDispatchToProps(action => dispatch({
      ...action,
      type: `${info.name}/${action.type}`,
    }), ownProps);
    return connect(_mapStateToProps, _mapDispatchToProps, mergeProps)(route);
  };
};
